@@ -29,7 +29,9 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <errno.h>
-#include <unistd.h>
+#ifndef _MSC_VER
+# include <unistd.h>
+#endif
 #include <ao/ao.h>
 #include <ao/plugin.h>
 
@@ -81,7 +83,7 @@ typedef struct ao_au_internal
 } ao_au_internal;
 
 
-static int ao_au_test()
+static int ao_au_test(void)
 {
 	return 1; /* File driver always works */
 }
